{% load static %}

<!DOCTYPE html>
<html>

    <head>

        <meta charset="utf-8"/>
        <meta name="viewport" content="width=device-width, initial-scale=1.0"/>

        <!-- CSS Imports -->
        <link href="{% static 'uniauth/css/bootstrap.min.css' %}" rel="stylesheet" type="text/css"/>
        <link href="{% static 'uniauth/css/fa-regular.min.css' %}" rel="stylesheet" type="text/css"/>
        <link href="{% static 'uniauth/css/fa-solid.min.css' %}" rel="stylesheet" type="text/css"/>
        <link href="{% static 'uniauth/css/font-awesome.min.css' %}" rel="stylesheet" type="text/css"/>
        <link href="{% static 'uniauth/css/theme.css' %}" rel="stylesheet" type="text/css"/>
        <link href="{% static 'uniauth/css/style.css' %}" rel="stylesheet" type="text/css"/>

        <!-- Shared headers -->
        {% block shared-head %}
        {% endblock %}

        <!-- Page specific headers -->
        {% block head %}
        {% endblock %}

        <!-- Custom Styling -->
        {% block style %}
        {% endblock %}

    </head>

    <body class="skin-3 top-navigation">

        <!-- Page content -->
        {% block body %}
        {% endblock %}

        <!-- JS Imports -->
        <script src="{% static 'uniauth/js/jquery-3.1.1.min.js' %}"></script>
        <script src="{% static 'uniauth/js/bootstrap.min.js' %}"></script>

        <!-- Custom JS -->
        <script>
        </script>

        <!-- Page specific JS -->
        <script>
            {% block script %}
            {% endblock %}
        </script>

    </body>

</html>
