{% autoescape off %}
Hello {{ user.email }},

Someone asked for a password reset at this email address. If it was you, please click on the below link:

{{ protocol }}://{{ domain }}{% url 'uniauth:password-reset-verify' uidb64=uid token=token %}{{ query_params }}
{% endautoescape %}
