{% extends "uniauth/base.html" %}

{% load static %}

{% block shared-head %}
<link rel="shortcut icon" href="{% static 'uniauth/img/favicon.ico' %}"/>
{% endblock %}

{% block body %}
<div id="wrapper">
    <div id="page-wrapper" class="lavender-bg">
        <div id="content-wrapper">
            <div id="top-background"></div>
            {% block content %}
            {% endblock %}
        </div>
    </div>
</div>
{% endblock %}
