/* Custom styling for uniauth app */

#add-email {
    display: inline-block;
    width: calc(100% - 55px);
}

.badge-success {
    background-color: #52bb56;
    color: #FFFFFF;
}

.badge-primary {
    background-color: #039cfd;
    colore: #FFFFFF;
}

.btn-action {
    background: #039cfd;
    border-color: #039cfd;
    color: #FFFFFF;
}

#cas-link {
    padding-left: 5px;
}

#cas-select {
    display: inline-block;
    margin-left: 14px;
    width: 150px;
}

#cas-select.only-cas-login {
    margin-left: 6px;
}

#cas-select-container {
    margin-bottom: 0px;
}

.centered-title {
    display: block;
    text-align: center;
}

#change-email {
    display: inline-block;
    width: calc(100% - 58px);
}

#content-wrapper {
    height: 100%;
    margin-left: -15px;
    overflow: auto;
    position: absolute;
    width: 100%;
}

#create-account-link {
    float: left;
}

#error-text {
    background-color: #fcf0ed;
    color: #a94442;
    display: block;
    padding-top: 10px;
    padding-bottom: 10px;
    text-align: center;
}

#forgot-password-link {
    float: right;
}

div.hr-line-dashed.only-cas-login {
    margin-top
}

button.icon-btn {
    background-color: transparent;
    border: 0px;
    cursor: pointer;
}

button.icon-btn i {
    font-size: 18px;
}

input.form-control:focus,
select.form-control:focus {
    border-color: #039cfd;
}

input.form-control::placeholder {
    color: #B0B0B0;
    opacity: 1;
}

#header-uniauth-text {
    font-weight: 600;
}

#id_password {
    margin-top: 5px;
}

#id_username {
    margin-top: 5px
}

#link-success-form-wrapper {
    left: calc(50% - 160px);
    width: 320px;
}

#link-to-profile-form-spacer {
    height: calc(40% - 162px);
}

#link-to-profile-form-wrapper form {
    padding-top: 10px;
}

#link-to-profile-title {
    padding-bottom: 10px;
}

#login-form-spacer {
    height: calc(40% - 147px);
}

#login-form-spacer.only-cas-login {
    height: calc(40% - 72px);
}

#login-form-wrapper.only-cas-login div.form-group {
    margin-bottom: 0px;
}

#login-title {
    display: block;
    font-family: Roboto;
    font-weight: 300;
    text-align: center;
}

#logout-form-wrapper {
    left: calc(50% - 140px);
    width: 280px;
}

#logout-followup {
    font-size: 13px;
}

#logout-text {
    font-size: 14px;
}

.no-margin-bottom {
    margin-bottom: 0px;
}

#password-reset-form-wrapper form {
    padding-top: 5px;
}

#password-reset-title {
    padding-bottom: 10px;
}

#password-reset-verify-success-form-spacer {
    height: calc(40% - 127px);
}

#password-reset-verify-success-form-wrapper #submit-btn {
    margin-top: 0px;
}

#password-reset-verify-success-title {
    padding-bottom: 10px;
}

@media(min--moz-device-pixel-ratio:0)  {
    div.form-group select.form-control:not([size]):not([multiple]) {
        height: auto;
    }
}

#settings-accounts-list {
    padding-bottom: 10px;
}

#settings-cas-select {
    display: inline-block;
    margin-left: 10px;
    width: 150px;
}

.settings-email-buttons {
    float: right;
    padding-top: 5px;
}

.settings-email-delete-btn {
    color: red;
}

#settings-form-spacer {
    height: 0px;
}

#settings-form-wrapper {
    left: calc(50% - 225px);
    width: 450px;
}

#settings-form-wrapper .hr-line-dashed {
    margin-bottom: 30px;
    margin-top: 30px;
}

#settings-form-wrapper li .badge {
    font-size: 12px;
    margin-left: 5px;
    padding: 8px;
}

#settings-form-wrapper label {
    font-weight: 700;
    margin-bottom: 5px;
}

.settings-header {
    display: block;
    font-size: 20px;
    font-weight: 600;
    padding-bottom: 10px;
}

.settings-item-text {
    display: inline-block;
    float: left;
    max-width: 225px;
    overflow: hidden;
    padding-top: 6px;
    padding-bottom: 6px;
    text-overflow: ellipsis;
}

.settings-password-row {
    padding-bottom: 10px;
}

.settings-password-row input {
    margin-bottom: 5px;
}

#signup-form-spacer {
    height: calc(40% - 185px);
}

#signup-form-wrapper {
    left: calc(50% - 200px);
    width: 400px;
}

#signup-form-wrapper #submit-btn {
    margin-top: 10px;
}

.signup-row {
    padding-bottom: 10px;
}

.signup-row label {
    font-weight: 700;
    margin-bottom: 0px;
}

.signup-row input {
    margin-top: 5px;
    margin-bottom: 5px;
}

.simple-title {
    color: #4c5667;
    font-size: 24px;
    font-weight: 600;
}

div.spacer {
    height: calc(40% - 60px);
    min-height: 20px;
}

#submit-btn {
    margin-top: 20px;
    width: 100%;
}

#success-text {
    background-color: #eafaee;
    color: #28a745;
    display: block;
    padding-top: 10px;
    padding-bottom: 10px;
    text-align: center;
}

.success-wrapper {
    margin-bottom: 0px;
    margin-top: 10px;
}

#top-background {
    background: #4c5667;
    height: 40%;
    position: fixed;
    top: 0px;
    width: 100%;
}

div.uniauth-wrapper.wrapper.wrapper-content {
    border-radius: 5px;
    box-shadow: 0px 2px 1px;
    left: calc(50% - 160px);
    margin-bottom: 20px;
    overflow: hidden;
    padding: 0;
    position: absolute;
    width: 320px;
}

.body-small div.uniauth-wrapper.wrapper.wrapper-content {
    padding: 0;
}

div.uniauth-wrapper div.ibox {
    margin-bottom: 0px;
}

#verify-failure-form-wrapper {
    left: calc(50% - 170px);
    width: 340px;
}

#verify-success-form-wrapper,
#verify-waiting-form-wrapper {
    left: calc(50% - 155px);
    width: 310px;
}


